@@ -1,6 +1,6 @@
 //  MIT License
 //
-//  Copyright (c) 2021 TileDB Inc.
+//  Copyright (c) 2021-2022 TileDB Inc.
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -20,9 +20,6 @@
 //  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 //  SOFTWARE.
 
-#define STRICT_R_HEADERS
-#include <Rcpp.h>
-
 #include "libtiledb.h"
 
 // In principle, the following functions could be templated. In practive it is a little
